@@ -17,14 +17,7 @@
 # along with Fail2Ban; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-# Author: Cyril Jaquier
-# Author: Arturo 'Buanzo' Busleiman
-# 
-# $Revision$
-
-__author__ = "Cyril Jaquier"
-__version__ = "$Revision$"
-__date__ = "$Date$"
+__author__ = "Cyril Jaquier, Arturo 'Buanzo' Busleiman"
 __copyright__ = "Copyright (c) 2009 Cyril Jaquier"
 __license__ = "GPL"
 
@@ -36,6 +29,12 @@ def formatExceptionInfo():
 	excName = cla.__name__
 	try:
 		excArgs = exc.__dict__["args"]
+		# Assure that we always return a string, without unneeded
+		# 'decorations' with python <= 2.5 where args would be a tuple
+		if isinstance(excArgs, tuple) and len(excArgs) == 1:
+			excArgs = excArgs[0]
+		excArgs = str(excArgs)
 	except KeyError:
+		# And always provide a string output
 		excArgs = str(exc)
 	return (excName, excArgs)
